#include <iostream>

#include "common.h"

#include "library/summation.hpp"
#include "library/seqinfileenumerator.hpp"

using namespace std;

class KisZhSum : public Summation<Item,int>
{
    std::string _akt;
protected:
    void init() override
    {
        _akt = _enor->current().neptun;
        *_result = 0;
    }
    void add(const Item& e) override
    {
        if(e.result == '+')
        {
            ++(*_result);
        }
        else if(e.result == '-')
        {
            --(*_result);
        }
        else
        {
            throw std::exception{};
        }
    }
    bool whileCond(const Item& e) const override
    {
        return _akt == e.neptun;
    }
    void first() override {}
};

class EredmenyEnumerator : public Enumerator<Result>
{
public:
    EredmenyEnumerator(const std::string& fname) : file(fname){}

protected:
    void first() override
    {
        file.first();
        next();
    }
    void next() override
    {
        _end = file.end();
        if(_end) return;
        res.neptun = file.current().neptun;
        KisZhSum sum;
        sum.addEnumerator(&file);
        sum.run();
        res.points = sum.result();
    }
    bool end() const override
    {
        return _end;
    }
    Result current() const override
    {
        return res;
    }

    SeqInFileEnumerator<Item> file;
    Result res;
    bool _end;
};

class StudentList : public Summation<Result,std::ofstream>
{
public:
    StudentList(std::string outfile)
    {
        _result->open(outfile.c_str());
    }
protected:
    void init(){}
    bool cond(const Result& e) const override
    {
        return e.points >= 0;
    }
    void add(const Result& e) override
    {
        (*_result) << e;
    }
};

int main()
{
    EredmenyEnumerator en("input.txt");
    StudentList ki("output.txt");
    ki.addEnumerator(&en);
    ki.run();
    return 0;
}
